@@ -860,12 +860,14 @@ simSCProfiles <- function(
   cell.types.used <- list.data[[2]][rownames(zinb.object@model@X), 
                                     cell.type.column] %>% unique()
   # generate metadata for simulated cells
-  colnames(list.data[[1]]) <- paste(list.data[[2]][, cell.type.column],
-                                    list.data[[2]][, cell.ID.column],
-                                    sep = "_") 
-  list.data[[2]]$simCellName <- paste(list.data[[2]][, cell.type.column],
-                                      list.data[[2]][, cell.ID.column],
-                                      sep = "_")
+  colnames(list.data[[1]]) <- paste(
+    list.data[[2]][, cell.type.column], 
+    list.data[[2]][, cell.ID.column], sep = "_"
+  ) 
+  list.data[[2]]$simCellName <- paste(
+    list.data[[2]][, cell.type.column],
+    list.data[[2]][, cell.ID.column], sep = "_"
+  )
   list.data[[2]]$Simulated <- FALSE
   # cell types in model
   cell.set.names <- NULL
@@ -1068,7 +1070,21 @@ simSCProfiles <- function(
   sim.cells.metadata$Simulated <- TRUE
   if (any(colnames(sim.cells.metadata) == "suffix"))
     warning("\n'suffix' column in cells metadata is going to be overwritten")
+  
   sim.cells.metadata$suffix <- suffix.names
+  if (any(apply(sim.counts, MARGIN = 2, FUN = sum) == 0)) {
+    warning(
+      paste(
+        "Some of the simulated cells have a library size equal to zero.", 
+        "This may be because the original library size of these cells is", 
+        "too low. They will be removed for further analysis, so check your", 
+        "initial parameters when loading scRNA-seq data"
+      )
+    )
+    pos.cells <- which(apply(sim.counts, MARGIN = 2, FUN = sum) != 0)
+    sim.counts <- sim.counts[, pos.cells]
+    sim.cells.metadata <- sim.cells.metadata[pos.cells, ]
+  }
   
   sim.sce <- .createSCEObject(
     counts = sim.counts,
